@@ -1,6 +1,6 @@
-/* $Id: gucharmap-charmap.c,v 1.35 2003/11/18 19:54:59 mariano Exp $ */
+/* $Id: gucharmap-charmap.c,v 1.46 2004/02/02 00:03:49 nlevitt Exp $ */
 /*
- * Copyright (c) 2003  Noah Levitt <nlevitt аt users.sourceforge.net>
+ * Copyright (c) 2004 Noah Levitt
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,35 +17,22 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include "config.h"
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <gucharmap/gucharmap.h>
-#include "gucharmap_intl.h"
-#include "gucharmap_marshal.h"
+#include "gucharmap-charmap.h"
+#include "gucharmap-unicode-info.h"
+#include "gucharmap-script-chapters.h"
+#include "gucharmap-intl.h"
+#include "gucharmap-marshal.h"
 #include "chartable_accessible.h"
 
-
-/* 0x100, a standard increment for paging unicode */
-#define PAGE_SIZE 256
-
 extern gboolean _gucharmap_unicode_has_nameslist_entry (gunichar uc);
 
-/* only the label is visible in the block selector */
-enum 
-{
-  BLOCK_SELECTOR_LABEL = 0,
-  BLOCK_SELECTOR_UC_START,
-  BLOCK_SELECTOR_UNICODE_BLOCK,
-  BLOCK_SELECTOR_NUM_COLUMNS
-};
-
 enum 
 {
   STATUS_MESSAGE = 0,
@@ -55,7 +42,6 @@ enum
 
 static guint gucharmap_charmap_signals[NUM_SIGNALS] = { 0, 0 };
 
-
 static void
 status_message (GucharmapCharmap *charmap, const gchar *message)
 {
@@ -63,155 +49,15 @@ status_message (GucharmapCharmap *charmap, const gchar *message)
                  0, message);
 }
 
-
-/* XXX: linear search (but N is small) */
-static GtkTreePath *
-find_block_index_tree_path (GucharmapCharmap *charmap, gunichar uc)
-{
-  gint i;
-
-  for (i = 0;  i < charmap->block_index_size;  i++)
-    if (charmap->block_index[i].start > uc)
-      break;
-
-  return charmap->block_index[i-1].tree_path;
-}
-
-
-/* selects the active block in the block selector tree view based on the
- * active character */
-static void
-set_active_block (GucharmapCharmap *charmap, gunichar uc)
-{
-  GtkTreePath *parent = NULL;
-  GtkTreePath *tree_path;
-  
-  tree_path = find_block_index_tree_path (charmap, uc);
-
-  /* block our "changed" handler */
-  g_signal_handler_block (G_OBJECT (charmap->block_selection), 
-                          charmap->block_selection_changed_handler_id);
-
-  gtk_tree_view_set_cursor (GTK_TREE_VIEW (charmap->block_selector_view),
-                            tree_path, NULL, FALSE);
-
-  g_signal_handler_unblock (G_OBJECT (charmap->block_selection),
-                            charmap->block_selection_changed_handler_id);
-
-  if (parent != NULL)
-    gtk_tree_path_free (parent);
-}
-
-
-static void
-block_selection_changed (GtkTreeSelection *selection, 
-                         gpointer user_data)
-{
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  GucharmapCharmap *charmap;
-  gunichar uc_start;
-
-  charmap = GUCHARMAP_CHARMAP (user_data);
-
-  if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    {
-      gtk_tree_model_get (model, &iter, BLOCK_SELECTOR_UC_START, 
-                          &uc_start, -1);
-
-      gucharmap_table_set_active_character (charmap->chartable, uc_start);
-    }
-}
-
-
-/* makes the list of unicode blocks and code points */
-static GtkWidget *
-make_unicode_block_selector (GucharmapCharmap *charmap)
-{
-  GtkWidget *scrolled_window;
-  GtkTreeIter iter;
-  GtkCellRenderer *cell;
-  GtkTreeViewColumn *column;
-  gint i, bi;
-
-  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), 
-                                       GTK_SHADOW_ETCHED_IN);
-
-  charmap->block_selector_model = gtk_tree_store_new (
-          BLOCK_SELECTOR_NUM_COLUMNS, G_TYPE_STRING, 
-          G_TYPE_UINT, G_TYPE_POINTER);
-
-  charmap->block_index_size = gucharmap_count_blocks (UNICHAR_MAX) + 1;
-  charmap->block_index = g_malloc (charmap->block_index_size 
-                                   * sizeof (GucharmapBlockIndex));
-  bi = 0;
-
-  for (i = 0;  gucharmap_unicode_blocks[i].start != (gunichar)(-1)
-               && gucharmap_unicode_blocks[i].start <= UNICHAR_MAX;  i++)
-    {
-      gtk_tree_store_append (charmap->block_selector_model, &iter, NULL);
-      gtk_tree_store_set (charmap->block_selector_model, &iter, 
-                          BLOCK_SELECTOR_LABEL, 
-                          _(gucharmap_unicode_blocks[i].name),
-                          BLOCK_SELECTOR_UC_START, 
-                          gucharmap_unicode_blocks[i].start,
-                          BLOCK_SELECTOR_UNICODE_BLOCK, 
-                          &(gucharmap_unicode_blocks[i]),
-                          -1);
-      charmap->block_index[bi].start = gucharmap_unicode_blocks[i].start;
-      charmap->block_index[bi].tree_path = gtk_tree_model_get_path (
-              GTK_TREE_MODEL (charmap->block_selector_model), &iter);
-      bi++;
-    }
-
-  /* terminate value that is bigger than the biggest character */
-  charmap->block_index[bi].start = UNICHAR_MAX + 1;
-  charmap->block_index[bi].tree_path = NULL;
-
-  /* we have the model, now make the view */
-  charmap->block_selector_view = gtk_tree_view_new_with_model (
-          GTK_TREE_MODEL (charmap->block_selector_model));
-  charmap->block_selection = gtk_tree_view_get_selection (
-          GTK_TREE_VIEW (charmap->block_selector_view));
-  gtk_tree_view_set_headers_visible (
-          GTK_TREE_VIEW (charmap->block_selector_view), FALSE);
-
-  cell = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes (NULL, cell, 
-                                                     "text", 0, NULL);
-
-  gtk_tree_view_append_column (GTK_TREE_VIEW (charmap->block_selector_view),
-                               GTK_TREE_VIEW_COLUMN (column));
-
-  gtk_tree_selection_set_mode (charmap->block_selection, GTK_SELECTION_BROWSE);
-  charmap->block_selection_changed_handler_id = g_signal_connect (
-          G_OBJECT (charmap->block_selection), "changed", 
-          G_CALLBACK (block_selection_changed), charmap);
-
-  gtk_container_add (GTK_CONTAINER (scrolled_window), 
-                     charmap->block_selector_view);
-
-  gtk_widget_show_all (scrolled_window);
-
-  return scrolled_window;
-}
-
-
 static void 
 charmap_finalize (GObject *object)
 {
   GucharmapCharmap *charmap = GUCHARMAP_CHARMAP (object);
-  g_free (charmap->block_index);
 
   gdk_cursor_unref (charmap->hand_cursor);
   gdk_cursor_unref (charmap->regular_cursor);
 }
 
-
 void
 gucharmap_charmap_class_init (GucharmapCharmapClass *clazz)
 {
@@ -222,19 +68,18 @@ gucharmap_charmap_class_init (GucharmapCharmapClass *clazz)
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET (GucharmapCharmapClass, status_message),
                     NULL, NULL, gucharmap_marshal_VOID__STRING, G_TYPE_NONE, 
-		    1, G_TYPE_STRING);
+                    1, G_TYPE_STRING);
 
   gucharmap_charmap_signals[LINK_CLICKED] =
       g_signal_new ("link-clicked", gucharmap_charmap_get_type (), 
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET (GucharmapCharmapClass, link_clicked),
                     NULL, NULL, gucharmap_marshal_VOID__UINT_UINT, G_TYPE_NONE, 
-		    2, G_TYPE_UINT, G_TYPE_UINT);
+                    2, G_TYPE_UINT, G_TYPE_UINT);
 
   G_OBJECT_CLASS (clazz)->finalize = charmap_finalize;
 }
 
-
 static void
 insert_vanilla_detail (GucharmapCharmap *charmap, 
                        GtkTextBuffer *buffer,
@@ -249,7 +94,6 @@ insert_vanilla_detail (GucharmapCharmap *charmap,
   gtk_text_buffer_insert (buffer, iter, "\n", -1);
 }
 
-
 /* makes a nice string and makes it a link to the character */
 static void
 insert_codepoint (GucharmapCharmap *charmap,
@@ -268,15 +112,13 @@ insert_codepoint (GucharmapCharmap *charmap,
                                     "underline", PANGO_UNDERLINE_SINGLE, 
                                     NULL);
   /* add one so that zero is the "nothing" value, since U+0000 is a character */
-  g_object_set_data (G_OBJECT (tag), "link_character", 
-                     GUINT_TO_POINTER (uc + 1));
+  g_object_set_data (G_OBJECT (tag), "link-character", GUINT_TO_POINTER (uc + 1));
 
   gtk_text_buffer_insert_with_tags (buffer, iter, str, -1, tag, NULL);
 
   g_free (str);
 }
 
-
 static void
 insert_chocolate_detail_codepoints (GucharmapCharmap *charmap,
                                     GtkTextBuffer *buffer,
@@ -320,7 +162,6 @@ find_codepoint (const gchar *str)
   return NULL;
 }
 
-
 static void
 insert_string_link_codepoints (GucharmapCharmap *charmap,
                                GtkTextBuffer *buffer,
@@ -388,7 +229,6 @@ insert_heading (GucharmapCharmap *charmap,
   gtk_text_buffer_insert (buffer, iter, "\n\n", -1);
 }
 
-
 static void
 conditionally_insert_canonical_decomposition (GucharmapCharmap *charmap, 
                                               GtkTextBuffer *buffer,
@@ -592,10 +432,9 @@ set_details (GucharmapCharmap *charmap,
 #endif /* #if ENABLE_UNIHAN */
 }
 
-
 static void
-active_char_set (GtkWidget *widget, 
-                 gunichar uc, 
+active_char_set (GtkWidget        *widget, 
+                 gunichar          wc, 
                  GucharmapCharmap *charmap)
 {
   GString *gs;
@@ -603,20 +442,19 @@ active_char_set (GtkWidget *widget,
   const gchar **temps;
   gint i;
 
-  set_active_block (charmap, uc);
-  set_details (charmap, uc);
+  set_details (charmap, wc);
 
   gs = g_string_new (NULL);
-  g_string_append_printf (gs, "U+%4.4X %s", uc, 
-                          gucharmap_get_unicode_name (uc));
+  g_string_append_printf (gs, "U+%4.4X %s", wc, 
+                          gucharmap_get_unicode_name (wc));
 
 #if ENABLE_UNIHAN
-  temp = gucharmap_get_unicode_kDefinition (uc);
+  temp = gucharmap_get_unicode_kDefinition (wc);
   if (temp)
     g_string_append_printf (gs, "   %s", temp);
 #endif
 
-  temps = gucharmap_get_nameslist_equals (uc);
+  temps = gucharmap_get_nameslist_equals (wc);
   if (temps)
     {
       g_string_append_printf (gs, "   = %s", temps[0]);
@@ -625,7 +463,7 @@ active_char_set (GtkWidget *widget,
       g_free (temps);
     }
 
-  temps = gucharmap_get_nameslist_stars (uc);
+  temps = gucharmap_get_nameslist_stars (wc);
   if (temps)
     {
       g_string_append_printf (gs, "   • %s", temps[0]);
@@ -638,7 +476,6 @@ active_char_set (GtkWidget *widget,
   g_string_free (gs, TRUE);
 }
 
-
 /* this creates all the named text tags we'll be using in set_details */
 static void
 create_tags (GucharmapCharmap *charmap)
@@ -649,15 +486,15 @@ create_tags (GucharmapCharmap *charmap)
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (charmap->details));
 
   default_font_size = pango_font_description_get_size (
-	  GTK_WIDGET (charmap)->style->font_desc);
+          GTK_WIDGET (charmap)->style->font_desc);
 
   gtk_text_buffer_create_tag (buffer, "gimongous", 
-	                      "size", 8 * default_font_size, 
+                              "size", 8 * default_font_size, 
                               "left-margin", PANGO_PIXELS (5 * default_font_size),
-			      NULL);
+                              NULL);
   gtk_text_buffer_create_tag (buffer, "bold",
-	                      "weight", PANGO_WEIGHT_BOLD,
-			      NULL);
+                              "weight", PANGO_WEIGHT_BOLD,
+                              NULL);
   gtk_text_buffer_create_tag (buffer, "big",
                               "size", default_font_size * 5 / 4,
                               NULL);
@@ -665,7 +502,6 @@ create_tags (GucharmapCharmap *charmap)
                               NULL);
 }
 
-
 static void
 follow_if_link (GucharmapCharmap *charmap,
                 GtkTextIter *iter)
@@ -679,13 +515,13 @@ follow_if_link (GucharmapCharmap *charmap,
       gunichar uc;
 
       /* subtract 1 because U+0000 is a character; see above where we set
-       * "link_character" */
-      uc = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (tag), "link_character")) - 1;
+       * "link-character" */
+      uc = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (tag), "link-character")) - 1;
 
       if (uc != (gunichar)(-1)) 
         {
           g_signal_emit (charmap, gucharmap_charmap_signals[LINK_CLICKED], 
-                          0, charmap->chartable->active_char, uc);
+                          0, charmap->chartable->active_cell, uc);
           gucharmap_table_set_active_character (charmap->chartable, uc);
           break;
         }
@@ -695,7 +531,6 @@ follow_if_link (GucharmapCharmap *charmap,
     g_slist_free (tags);
 }
 
-
 static gboolean
 details_key_press_event (GtkWidget *text_view,
                          GdkEventKey *event,
@@ -721,7 +556,6 @@ details_key_press_event (GtkWidget *text_view,
   return FALSE;
 }
 
-
 static gboolean
 details_event_after (GtkWidget *text_view,
                      GdkEvent *ev,
@@ -758,7 +592,6 @@ details_event_after (GtkWidget *text_view,
   return FALSE;
 }
 
-
 static void
 set_cursor_if_appropriate (GucharmapCharmap *charmap,
                            gint x,
@@ -781,8 +614,8 @@ set_cursor_if_appropriate (GucharmapCharmap *charmap,
       gunichar uc;
 
       /* subtract 1 because U+0000 is a character; see above where we set
-       * "link_character" */
-      uc = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (tag), "link_character")) - 1;
+       * "link-character" */
+      uc = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (tag), "link-character")) - 1;
 
       if (uc != (gunichar)(-1)) 
         {
@@ -805,7 +638,6 @@ set_cursor_if_appropriate (GucharmapCharmap *charmap,
     g_slist_free (tags);
 }
 
-
 static gboolean
 details_motion_notify_event (GtkWidget *text_view,
                              GdkEventMotion *event,
@@ -823,7 +655,6 @@ details_motion_notify_event (GtkWidget *text_view,
   return FALSE;
 }
 
-
 static gboolean
 details_visibility_notify_event (GtkWidget *text_view,
                                  GdkEventVisibility *event,
@@ -842,11 +673,10 @@ details_visibility_notify_event (GtkWidget *text_view,
   return FALSE;
 }
 
-
 static GtkWidget *
 make_details_page (GucharmapCharmap *charmap)
 {
-  GtkWidget *scrolled_window;
+  GtkWidget *scrolled_window = NULL;
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_show (scrolled_window);
@@ -877,9 +707,9 @@ make_details_page (GucharmapCharmap *charmap)
   return scrolled_window;
 }
 
-
 static GtkWidget *
-make_chartable_pane (GucharmapCharmap *charmap)
+make_chartable_pane (GucharmapCharmap       *charmap,
+                     GucharmapCodepointList *codepoint_list)
 {
   GtkWidget *notebook;
 
@@ -887,6 +717,8 @@ make_chartable_pane (GucharmapCharmap *charmap)
   gtk_widget_show (notebook);
 
   charmap->chartable = GUCHARMAP_TABLE (gucharmap_table_new ());
+  gucharmap_table_set_codepoint_list (charmap->chartable, codepoint_list);
+
   gtk_widget_show (GTK_WIDGET (charmap->chartable));
   g_signal_connect (G_OBJECT (charmap->chartable), "set-active-char", 
                     G_CALLBACK (active_char_set), charmap);
@@ -903,37 +735,40 @@ make_chartable_pane (GucharmapCharmap *charmap)
   return notebook;
 }
 
+static void
+chapter_changed (GucharmapChapters *chapters,
+                 GucharmapCharmap  *charmap)
+{
+  gucharmap_table_set_codepoint_list (charmap->chartable, gucharmap_chapters_get_codepoint_list (chapters));
+}
 
-void
+static void
 gucharmap_charmap_init (GucharmapCharmap *charmap)
 {
-  AtkObject *accessib;
-  GtkWidget *block_selector;
+}
+
+GtkWidget *
+gucharmap_charmap_new (GucharmapChapters *chapters)
+{
+  GucharmapCharmap *charmap = g_object_new (gucharmap_charmap_get_type (), NULL);
+  GtkWidget *pane2;
 
   charmap->hand_cursor = gdk_cursor_new (GDK_HAND2);
   charmap->regular_cursor = gdk_cursor_new (GDK_XTERM);
   charmap->hovering_over_link = FALSE;
+  gtk_widget_show (GTK_WIDGET (chapters));
 
-  block_selector = make_unicode_block_selector (charmap);
-  accessib = gtk_widget_get_accessible (block_selector);
-  atk_object_set_name (accessib, _("List of Unicode Blocks"));
+  g_signal_connect (G_OBJECT (chapters), "changed", G_CALLBACK (chapter_changed), charmap);
 
-  gtk_paned_pack1 (GTK_PANED (charmap), block_selector, FALSE, TRUE);
-  gtk_paned_pack2 (GTK_PANED (charmap), make_chartable_pane (charmap), 
-                   TRUE, TRUE);
+  pane2 = make_chartable_pane (charmap, gucharmap_chapters_get_codepoint_list (GUCHARMAP_CHAPTERS (chapters)));
+  gtk_paned_pack1 (GTK_PANED (charmap), GTK_WIDGET (chapters), FALSE, TRUE);
+  gtk_paned_pack2 (GTK_PANED (charmap), pane2, TRUE, TRUE);
 
-  set_active_block (charmap, charmap->chartable->active_char);
-  set_details (charmap, charmap->chartable->active_char);
+  set_details (charmap, gucharmap_table_get_active_character (charmap->chartable));
+
+  return GTK_WIDGET (charmap);
 }
 
-
-GtkWidget *
-gucharmap_charmap_new (void)
-{
-  return GTK_WIDGET (g_object_new (gucharmap_charmap_get_type (), NULL));
-}
-
-
 GType
 gucharmap_charmap_get_type (void)
 {
@@ -971,75 +806,40 @@ gucharmap_charmap_set_font (GucharmapCharmap *charmap,
   gucharmap_table_set_font (charmap->chartable, font_name);
 }
 
-
-void
-gucharmap_charmap_identify_clipboard (GucharmapCharmap *charmap, 
-                                      GtkClipboard *clipboard)
-{
-  gucharmap_table_identify_clipboard (charmap->chartable, clipboard);
-}
-
-
 void
 gucharmap_charmap_go_to_character (GucharmapCharmap *charmap, 
-                                   gunichar uc)
+                                   gunichar          wc)
 {
-  if (uc >= 0 && uc <= UNICHAR_MAX)
-    gucharmap_table_set_active_character (charmap->chartable, uc);
+  GucharmapChapters *chapters = gucharmap_charmap_get_chapters (charmap);
+  gboolean status;
+
+  status = gucharmap_chapters_go_to_character (chapters, wc);
+  if (!status)
+    g_warning ("gucharmap_chapters_go_to_character failed (%04X)\n", wc);
+
+  if (wc >= 0 && wc <= UNICHAR_MAX)
+    gucharmap_table_set_active_character (charmap->chartable, wc);
 }
 
-
-/* direction is +1 (forward) or -1 (backward) */
-GucharmapSearchResult
-gucharmap_charmap_search (GucharmapCharmap *charmap, 
-                          const gchar *search_text, 
-                          gint direction)
-{
-  gunichar uc;
-  GucharmapSearchResult result;
-
-  g_assert (direction == -1 || direction == 1);
-
-  if (search_text[0] == '\0')
-    return GUCHARMAP_NOTHING_TO_SEARCH_FOR;
-  
-  uc = gucharmap_find_substring_match (charmap->chartable->active_char, 
-                             search_text, direction);
-  if (uc != (gunichar)(-1) && uc <= UNICHAR_MAX)
-    {
-      if ((direction == 1 && uc <= charmap->chartable->active_char)
-          || (direction == -1 && uc >= charmap->chartable->active_char))
-        result = GUCHARMAP_WRAPPED;
-      else
-        result = GUCHARMAP_FOUND;
-
-      gucharmap_table_set_active_character (charmap->chartable, uc);
-    }
-  else
-    result = GUCHARMAP_NOT_FOUND;
-
-  return result;
-}
-
-
-void
-gucharmap_charmap_zoom_enable (GucharmapCharmap *charmap)
-{
-  gucharmap_table_zoom_enable (charmap->chartable);
-}
-
-
-void
-gucharmap_charmap_zoom_disable (GucharmapCharmap *charmap)
-{
-  gucharmap_table_zoom_disable (charmap->chartable);
-}
-
-
 GucharmapTable *
 gucharmap_charmap_get_chartable (GucharmapCharmap *charmap)
 {
   return charmap->chartable;
 }
 
+void
+gucharmap_charmap_set_chapters (GucharmapCharmap  *charmap,
+                                GucharmapChapters *chapters)
+{
+  gtk_container_remove (GTK_CONTAINER (charmap), GTK_PANED (charmap)->child1);
+  gtk_paned_pack1 (GTK_PANED (charmap), GTK_WIDGET (chapters), FALSE, TRUE);
+  gucharmap_table_set_codepoint_list (charmap->chartable, gucharmap_chapters_get_codepoint_list (chapters));
+  g_signal_connect (G_OBJECT (chapters), "changed", G_CALLBACK (chapter_changed), charmap);
+  gtk_widget_show (GTK_WIDGET (chapters));
+}
 
+GucharmapChapters *
+gucharmap_charmap_get_chapters (GucharmapCharmap  *charmap)
+{
+  return GUCHARMAP_CHAPTERS (GTK_PANED (charmap)->child1);
+}
