@@ -1,4 +1,4 @@
-/* $Id: gucharmap-search-dialog.c,v 1.9 2004/02/02 19:57:11 nlevitt Exp $ */
+/* $Id: gucharmap-search-dialog.c,v 1.12 2004/02/25 22:15:38 nlevitt Exp $ */
 /*
  * Copyright (c) 2004 Noah Levitt
  *
@@ -29,6 +29,8 @@
 
 #define GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), gucharmap_search_dialog_get_type (), GucharmapSearchDialogPrivate))
 
+extern GdkCursor * _gucharmap_window_progress_cursor ();
+
 enum
 {
   SEARCH_START,
@@ -50,6 +52,7 @@ typedef struct _GucharmapSearchState GucharmapSearchState;
 struct _GucharmapSearchState
 {
   GucharmapCodepointList *list;
+  gchar                  *search_string;
   gchar                  *search_string_nfd;
   const gchar            *no_leading_space;  /* points into search_string_nfd */
   gint                    start_index;
@@ -193,7 +196,7 @@ check_for_explicit_codepoint (const GucharmapCodepointList *list,
 }
 
 static gboolean
-quick_checks (GucharmapSearchState *search_state)
+quick_checks_before (GucharmapSearchState *search_state)
 {
   gint index;
 
@@ -232,15 +235,66 @@ quick_checks (GucharmapSearchState *search_state)
   return FALSE;
 }
 
+static gboolean
+quick_checks_after (GucharmapSearchState *search_state)
+{
+  gchar *search_string_nfc;
+  gchar *endptr;
+  gunichar wc;
+  
+  /* if NFC of the search string is a single character, jump to that */
+  search_string_nfc = g_utf8_normalize (search_state->no_leading_space, -1, G_NORMALIZE_NFC);
+  if (g_utf8_strlen (search_string_nfc, -1) == 1)
+    {
+      gint index = gucharmap_codepoint_list_get_index (search_state->list, g_utf8_get_char (search_string_nfc));
+      g_free (search_string_nfc);
+
+      if (index != -1)
+        {
+          search_state->found_index = index;
+          search_state->dont_search = TRUE;
+          return TRUE;
+        }
+    }
+  else
+    g_free (search_string_nfc);
+
+  /* jump to the first nonspace character unless it’s plain ascii */
+  if (*search_state->no_leading_space < 0x20 || *search_state->no_leading_space > 0x7e)
+    {
+      gint index = gucharmap_codepoint_list_get_index (search_state->list, g_utf8_get_char (search_state->no_leading_space));
+      if (index != -1)
+        {
+          search_state->found_index = index;
+          search_state->dont_search = TRUE;
+          return TRUE;
+        }
+    }
+
+  /* check for hex codepoint without any prefix */
+  wc = strtoul (search_state->no_leading_space, &endptr, 16);
+  if (endptr != search_state->no_leading_space)
+    {
+      gint index = gucharmap_codepoint_list_get_index (search_state->list, wc);
+      if (index != -1)
+        {
+          search_state->found_index = index;
+          search_state->dont_search = TRUE;
+          return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+
 static gboolean
 idle_search (GucharmapSearchDialog *search_dialog)
 {
   GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
   GTimer *timer = g_timer_new ();
   gunichar wc;
-  gint index;
 
-  if (quick_checks (priv->search_state))
+  if (quick_checks_before (priv->search_state))
     return FALSE;
 
   /* XXX: search with leading spaces? */
@@ -270,16 +324,8 @@ idle_search (GucharmapSearchDialog *search_dialog)
     }
   while (priv->search_state->curr_index != priv->search_state->start_index);
 
-  /* jump to the first nonspace character unless it’s plain ascii */
-  if (*priv->search_state->no_leading_space < 0x20 || *priv->search_state->no_leading_space > 0x7e)
-    {
-      index = gucharmap_codepoint_list_get_index (priv->search_state->list, g_utf8_get_char (priv->search_state->no_leading_space));
-      if (index != -1)
-        {
-          priv->search_state->found_index = index;
-          priv->search_state->dont_search = TRUE;
-        }
-    }
+  if (quick_checks_after (priv->search_state))
+    return FALSE;
 
   priv->search_state->dont_search = TRUE;
   return FALSE;
@@ -290,7 +336,7 @@ idle_search (GucharmapSearchDialog *search_dialog)
  * @search_state: 
  * Return value: 
  **/
-gunichar
+static gunichar
 gucharmap_search_state_get_found_char (GucharmapSearchState *search_state)
 {
   if (search_state->found_index > 0)
@@ -303,7 +349,7 @@ gucharmap_search_state_get_found_char (GucharmapSearchState *search_state)
  * gucharmap_search_state_free:
  * @search_state: 
  **/
-void
+static void
 gucharmap_search_state_free (GucharmapSearchState *search_state)
 {
   g_free (search_state->search_string_nfd);
@@ -323,7 +369,7 @@ gucharmap_search_state_free (GucharmapSearchState *search_state)
  *
  * Return value: the new #GucharmapSearchState.
  **/
-GucharmapSearchState * 
+static GucharmapSearchState * 
 gucharmap_search_state_new (const GucharmapCodepointList *list, 
                             const gchar                  *search_string, 
                             gint                          start_index, 
@@ -339,6 +385,7 @@ gucharmap_search_state_new (const GucharmapCodepointList *list,
   search_state->list = (GucharmapCodepointList *) list;
   search_state->list_num_chars = gucharmap_codepoint_list_get_last_index (search_state->list) + 1;
 
+  search_state->search_string = g_strdup (search_string);
   search_state->search_string_nfd = g_utf8_normalize (search_string, -1, G_NORMALIZE_NFD);
 
   search_state->increment = direction;
@@ -432,17 +479,31 @@ gucharmap_search_dialog_start_search (GucharmapSearchDialog *search_dialog,
   gunichar start_char;
   gint start_index;
 
-  GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
+  GdkCursor *cursor = _gucharmap_window_progress_cursor ();
   gdk_window_set_cursor (GTK_WIDGET (search_dialog)->window, cursor);
   gdk_cursor_unref (cursor);
 
-  if (priv->search_state)
-    gucharmap_search_state_free (priv->search_state);
 
   list = (GucharmapCodepointList *) gucharmap_chapters_get_book_codepoint_list (gucharmap_charmap_get_chapters (priv->guw->charmap));
-  start_char = gucharmap_table_get_active_character (priv->guw->charmap->chartable);
-  start_index = gucharmap_codepoint_list_get_index (list, start_char);
-  priv->search_state = gucharmap_search_state_new (list, gtk_entry_get_text (GTK_ENTRY (priv->entry)), start_index, direction, FALSE);
+
+  if (priv->search_state == NULL
+      || list != priv->search_state->list
+      || strcmp (priv->search_state->search_string, gtk_entry_get_text (GTK_ENTRY (priv->entry))) != 0)
+    {
+      if (priv->search_state)
+        gucharmap_search_state_free (priv->search_state);
+
+      start_char = gucharmap_table_get_active_character (priv->guw->charmap->chartable);
+      start_index = gucharmap_codepoint_list_get_index (list, start_char);
+      priv->search_state = gucharmap_search_state_new (list, gtk_entry_get_text (GTK_ENTRY (priv->entry)), start_index, direction, FALSE);
+    }
+  else
+    {
+      start_char = gucharmap_table_get_active_character (priv->guw->charmap->chartable);
+      priv->search_state->start_index = gucharmap_codepoint_list_get_index (list, start_char);
+      priv->search_state->curr_index = priv->search_state->start_index;
+      priv->search_state->increment = direction;
+    }
 
   gtk_widget_set_sensitive (priv->prev_button, FALSE);
   gtk_widget_set_sensitive (priv->next_button, FALSE);
@@ -459,6 +520,7 @@ search_find_response (GtkDialog *dialog,
                       gint       response)
 {
   GucharmapSearchDialog *search_dialog = GUCHARMAP_SEARCH_DIALOG (dialog);
+  GucharmapSearchDialogPrivate *priv = GUCHARMAP_SEARCH_DIALOG_GET_PRIVATE (search_dialog);
 
   switch (response)
     {
@@ -474,6 +536,8 @@ search_find_response (GtkDialog *dialog,
         gtk_widget_hide (GTK_WIDGET (search_dialog));
         break;
     }
+
+  gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
 }
 
 static void
