@@ -1,6 +1,6 @@
-/* $Id: gucharmap-unicode-info.c,v 1.4 2003/06/09 05:15:22 nlevitt Exp $ */
+/* $Id: gucharmap-unicode-info.c,v 1.11 2004/02/01 05:43:22 nlevitt Exp $ */
 /*
- * Copyright (c) 2003 Noah Levitt
+ * Copyright (c) 2004 Noah Levitt
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,82 +17,20 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  */
 
-#if HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include "config.h"
 
 #include <gtk/gtk.h>
 #include <string.h>
-#include <gucharmap_intl.h>
-#include <gucharmap/gucharmap-unicode-info.h>
+#include "gucharmap-intl.h"
+#include "gucharmap-unicode-info.h"
 
-
-typedef struct 
-{
-  gunichar index;
-  const gchar *name;
-} 
-UnicodeData;
-
-
-typedef struct 
-{
-  gunichar index;
-  const gchar *kDefinition;
-  const gchar *kCantonese;
-  const gchar *kMandarin;
-  const gchar *kTang;
-  const gchar *kKorean;
-  const gchar *kJapeneseKun;
-  const gchar *kJapaneseOn;
-} 
-Unihan;
-
-
-
-typedef struct 
-{
-  gunichar index;
-  gchar *value;
-}
-UnicharString;
-
-typedef struct
-{
-  gunichar index;
-  gunichar value;
-}
-UnicharUnichar;
-
-typedef struct
-{
-  gunichar index;
-  gint equals_index;  /* -1 means */
-  gint stars_index;   /* this character */
-  gint exes_index;    /* doesn't */
-  gint pounds_index;  /* have any */
-  gint colons_index;
-}
-NamesList;
-
-
-typedef struct
-{
-  gunichar first;
-  gunichar last;
-  GUnicodeType category;
-}
-UnicodeCategory;
-
-
-#include "unicode/unicode_data.cI"
-#include "unicode/unicode_blocks.cI"
+#include "unicode-names.h"
+#include "unicode-blocks.h"
+#include "unicode-nameslist.h"
+#include "unicode-categories.h"
 #if ENABLE_UNIHAN
-# include "unicode/unicode_unihan.cI"
+# include "unicode-unihan.h"
 #endif
-#include "unicode/unicode_nameslist.cI"
-#include "unicode/unicode_categories.cI"
-
 
 /* constants for hangul (de)composition, see UAX #15 */
 #define SBase 0xAC00
@@ -122,54 +60,51 @@ static const gchar * const JAMO_T_TABLE[] = {
   "S", "SS", "NG", "J", "C", "K", "T", "P", "H"
 };
 
-/* computes the hangul name as per UAX #15 */
 G_CONST_RETURN gchar *
-get_hangul_syllable_name (gunichar s)
+gucharmap_get_unicode_name (gunichar wc)
 {
   static gchar buf[32];
-  gint SIndex = s - SBase;
-  gint LIndex, VIndex, TIndex;
 
-  if (SIndex < 0 || SIndex >= SCount)
-    return "";
+  if ((wc >= 0x3400 && wc <= 0x4DB5) 
+      || (wc >= 0x4e00 && wc <= 0x9fa5) 
+      || (wc >= 0x20000 && wc <= 0x2A6D6))
+    {
+      g_snprintf (buf, sizeof (buf), "CJK UNIFIED IDEOGRAPH-%04X", wc);
+      return buf;
+    }
+  else if (wc >= 0xac00 && wc <= 0xd7af)
+    {
+      /* compute hangul syllable name as per UAX #15 */
+      gint SIndex = wc - SBase;
+      gint LIndex, VIndex, TIndex;
 
-  LIndex = SIndex / NCount;
-  VIndex = (SIndex % NCount) / TCount;
-  TIndex = SIndex % TCount;
+      if (SIndex < 0 || SIndex >= SCount)
+        return "";
 
-  g_snprintf (buf, 32, _("HANGUL SYLLABLE %s%s%s"), JAMO_L_TABLE[LIndex],
-              JAMO_V_TABLE[VIndex], JAMO_T_TABLE[TIndex]);
+      LIndex = SIndex / NCount;
+      VIndex = (SIndex % NCount) / TCount;
+      TIndex = SIndex % TCount;
 
-  return buf;
-}
+      g_snprintf (buf, sizeof (buf), "HANGUL SYLLABLE %s%s%s", 
+                  JAMO_L_TABLE[LIndex], JAMO_V_TABLE[VIndex], JAMO_T_TABLE[TIndex]);
 
-
-G_CONST_RETURN gchar *
-gucharmap_get_unicode_name (gunichar uc)
-{
-  if (uc >= 0x3400 && uc <= 0x4DB5)
-    return _("<CJK Ideograph Extension A>");
-  else if (uc >= 0x4e00 && uc <= 0x9fa5)
-    return _("<CJK Ideograph>");
-  else if (uc >= 0xac00 && uc <= 0xd7af)
-    return get_hangul_syllable_name (uc);
-  else if (uc >= 0xD800 && uc <= 0xDB7F) 
+      return buf;
+    }
+  else if (wc >= 0xD800 && wc <= 0xDB7F) 
     return _("<Non Private Use High Surrogate>");
-  else if (uc >= 0xDB80 && uc <= 0xDBFF) 
+  else if (wc >= 0xDB80 && wc <= 0xDBFF) 
     return _("<Private Use High Surrogate>");
-  else if (uc >= 0xDC00 && uc <= 0xDFFF)
-    return _("<Low Surrogate, Last>");
-  else if (uc >= 0xE000 && uc <= 0xF8FF) 
+  else if (wc >= 0xDC00 && wc <= 0xDFFF)
+    return _("<Low Surrogate>");
+  else if (wc >= 0xE000 && wc <= 0xF8FF) 
     return _("<Private Use>");
-  else if (uc >= 0xF0000 && uc <= 0xFFFFD)
+  else if (wc >= 0xF0000 && wc <= 0xFFFFD)
     return _("<Plane 15 Private Use>");
-  else if (uc >= 0x100000 && uc <= 0x10FFFD)
+  else if (wc >= 0x100000 && wc <= 0x10FFFD)
     return _("<Plane 16 Private Use>");
-  else if (uc >= 0x20000 && uc <= 0x2A6D6)
-    return _("<CJK Ideograph Extension B>");
   else
     {
-      const gchar *x = gucharmap_get_unicode_data_name (uc);
+      const gchar *x = gucharmap_get_unicode_data_name (wc);
       if (x == NULL)
         return _("<not assigned>");
       else
@@ -177,11 +112,10 @@ gucharmap_get_unicode_name (gunichar uc)
     }
 }
 
-
 G_CONST_RETURN gchar *
-gucharmap_get_unicode_category_name (gunichar uc)
+gucharmap_get_unicode_category_name (gunichar wc)
 {
-  switch (gucharmap_unichar_type (uc))
+  switch (gucharmap_unichar_type (wc))
     {
       case G_UNICODE_CONTROL: return _("Other, Control");
       case G_UNICODE_FORMAT: return _("Other, Format");
@@ -217,20 +151,6 @@ gucharmap_get_unicode_category_name (gunichar uc)
     }
 }
 
-
-/* counts the number of entries in gucharmap_unicode_blocks with start <= max */
-gint 
-gucharmap_count_blocks (gunichar max)
-{
-  gint i;
-
-  for (i = 0;  gucharmap_unicode_blocks[i].start != (gunichar)(-1)
-               && gucharmap_unicode_blocks[i].start < max;  i++);
-
-  return i;
-}
-
-
 /* http://www.unicode.org/unicode/reports/tr15/#Hangul */
 static gunichar *
 hangul_decomposition (gunichar s, gsize *result_len)
@@ -265,7 +185,6 @@ hangul_decomposition (gunichar s, gsize *result_len)
   return r;
 }
 
-
 /*
  * See http://bugzilla.gnome.org/show_bug.cgi?id=100456
  *
@@ -282,141 +201,52 @@ gunichar *
 gucharmap_unicode_canonical_decomposition (gunichar ch, 
                                            gsize *result_len)
 {
-  if (ch >= 0xac00 && ch <= 0xd7af)  /* Hangul syllable */
-    return hangul_decomposition (ch, result_len);
-  else 
+  if (GLIB_CHECK_VERSION (2,3,1))
     return g_unicode_canonical_decomposition (ch, result_len);
+  else
+    if (ch >= 0xac00 && ch <= 0xd7af)  /* Hangul syllable */
+      return hangul_decomposition (ch, result_len);
+    else 
+      return g_unicode_canonical_decomposition (ch, result_len);
 }
 
-
-
-/* does a binary search on unicode_data */
+/* does a binary search on unicode_names */
 G_CONST_RETURN gchar *
 gucharmap_get_unicode_data_name (gunichar uc)
 {
   gint min = 0;
   gint mid;
-  gint max = sizeof (unicode_data) / sizeof (UnicodeData) - 1;
+  gint max = sizeof (unicode_names) / sizeof (UnicodeName) - 1;
 
-  if (uc < unicode_data[0].index || uc > unicode_data[max].index)
+  if (uc < unicode_names[0].index || uc > unicode_names[max].index)
     return "";
 
   while (max >= min) 
     {
       mid = (min + max) / 2;
-      if (uc > unicode_data[mid].index)
+      if (uc > unicode_names[mid].index)
         min = mid + 1;
-      else if (uc < unicode_data[mid].index)
+      else if (uc < unicode_names[mid].index)
         max = mid - 1;
       else
-        return unicode_data[mid].name;
+        return unicode_names[mid].name;
     }
 
   return NULL;
 }
 
-
-/* ascii case-insensitive substring search (source ripped from glib) */
-static G_CONST_RETURN gchar *
-ascii_case_strrstr (const gchar *haystack, const gchar *needle)
+gint
+gucharmap_get_unicode_data_name_count ()
 {
-  gsize i;
-  gsize needle_len;
-  gsize haystack_len;
-  const gchar *p;
-      
-  g_return_val_if_fail (haystack != NULL, NULL);
-  g_return_val_if_fail (needle != NULL, NULL);
-
-  needle_len = strlen (needle);
-  haystack_len = strlen (haystack);
-
-  if (needle_len == 0)
-    return haystack;
-
-  if (haystack_len < needle_len)
-    return NULL;
-  
-  p = haystack + haystack_len - needle_len;
-
-  while (p >= haystack)
-    {
-      for (i = 0; i < needle_len; i++)
-        if (g_ascii_tolower (p[i]) != g_ascii_tolower (needle[i]))
-          goto next;
-      
-      return p;
-      
-    next:
-      p--;
-    }
-  
-  return NULL;
+  return G_N_ELEMENTS (unicode_names);
 }
 
-
-/* case insensitive; returns (gunichar)(-1) if nothing found */
-/* direction must be +1 or -1 */
-gunichar
-gucharmap_find_substring_match (gunichar start, 
-                                const gchar *search_text,
-                                gint direction)
+gint
+gucharmap_get_unihan_count ()
 {
-  gint max = sizeof (unicode_data) / sizeof (UnicodeData) - 1;
-  gint i0;
-  gint i;
-
-  g_assert (direction == -1 || direction == 1);
-
-  /* locate the start character by binary search */
-  if (start < unicode_data[0].index || start > UNICHAR_MAX)
-    i0 = 0;
-  else
-    {
-      gint min = 0, mid = 0;
-
-      while (max >= min) 
-        {
-          mid = (min + max) / 2;
-          if (start > unicode_data[mid].index)
-            min = mid + 1;
-          else if (start < unicode_data[mid].index)
-            max = mid - 1;
-          else
-            break;
-        }
-
-      i0 = mid;
-    }
-
-  max = sizeof (unicode_data) / sizeof (UnicodeData) - 1;
-  /* try substring match on each */
-  for (i = i0 + direction;  i != i0;  )
-    {
-      if (unicode_data[i].index > UNICHAR_MAX)
-        {
-          i += direction;
-          continue;
-        }
-
-      if (ascii_case_strrstr (unicode_data[i].name, search_text) != NULL)
-        return unicode_data[i].index;
-
-      i += direction;
-      if (i > max)
-        i = 0;
-      else if (i < 0)
-        i = max;
-    }
-
-  /* if the start character matches we want to return a match */
-  if (ascii_case_strrstr (unicode_data[i].name, search_text) != NULL)
-    return unicode_data[i].index;
-
-  return (gunichar)(-1);
+  return G_N_ELEMENTS (unihan);
 }
 
-
 #if ENABLE_UNIHAN
 
 /* does a binary search; also caches most recent, since it will often be
@@ -456,7 +286,6 @@ _get_unihan (gunichar uc)
   return NULL;
 }
 
-
 G_CONST_RETURN gchar * 
 gucharmap_get_unicode_kDefinition (gunichar uc)
 {
@@ -573,7 +402,6 @@ gucharmap_get_unicode_kJapaneseOn (gunichar uc)
 
 #endif /* #else (#if ENABLE_UNIHAN) */
 
-
 /* does a binary search; also caches most recent, since it will often be
  * called in succession on the same character */
 static G_CONST_RETURN NamesList *
@@ -611,7 +439,6 @@ get_nameslist (gunichar uc)
   return NULL;
 }
 
-
 gboolean
 _gucharmap_unicode_has_nameslist_entry (gunichar uc)
 {
@@ -643,7 +470,6 @@ gucharmap_get_nameslist_exes (gunichar uc)
   return exes;
 }
 
-
 /* returns newly allocated null-terminated array of gchar* */
 /* the items are const, but the array should be freed by the caller */
 G_CONST_RETURN gchar **
@@ -670,7 +496,6 @@ gucharmap_get_nameslist_equals (gunichar uc)
   return equals;
 }
 
-
 /* returns newly allocated null-terminated array of gchar* */
 /* the items are const, but the array should be freed by the caller */
 G_CONST_RETURN gchar **
@@ -697,7 +522,6 @@ gucharmap_get_nameslist_stars (gunichar uc)
   return stars;
 }
 
-
 /* returns newly allocated null-terminated array of gchar* */
 /* the items are const, but the array should be freed by the caller */
 G_CONST_RETURN gchar **
@@ -724,7 +548,6 @@ gucharmap_get_nameslist_pounds (gunichar uc)
   return pounds;
 }
 
-
 /* returns newly allocated null-terminated array of gchar* */
 /* the items are const, but the array should be freed by the caller */
 G_CONST_RETURN gchar **
@@ -751,7 +574,6 @@ gucharmap_get_nameslist_colons (gunichar uc)
   return colons;
 }
 
-
 #define UNICODE_VALID(Char)                   \
     ((Char) < 0x110000 &&                     \
     ((Char) < 0xD800 || (Char) >= 0xE000) &&  \
@@ -765,7 +587,6 @@ gucharmap_unichar_validate (gunichar ch)
   return UNICODE_VALID (ch);
 }
 
-
 /**
  * gucharmap_unichar_to_printable_utf8
  * @uc: a unicode character 
@@ -819,7 +640,6 @@ gucharmap_unichar_to_printable_utf8 (gunichar uc, gchar *outbuf)
     return g_unichar_to_utf8 (uc, outbuf);
 }
 
-
 /**
  * gucharmap_unichar_type:
  * @c: a Unicode character
@@ -835,15 +655,15 @@ gucharmap_unichar_type (gunichar uc)
   gint mid;
   gint max = sizeof (unicode_categories) / sizeof (UnicodeCategory) - 1;
 
-  if (uc < unicode_categories[0].first || uc > unicode_categories[max].last)
+  if (uc < unicode_categories[0].start || uc > unicode_categories[max].end)
     return G_UNICODE_UNASSIGNED;
 
   while (max >= min) 
     {
       mid = (min + max) / 2;
-      if (uc > unicode_categories[mid].last)
+      if (uc > unicode_categories[mid].end)
         min = mid + 1;
-      else if (uc < unicode_categories[mid].first)
+      else if (uc < unicode_categories[mid].start)
         max = mid - 1;
       else
         return unicode_categories[mid].category;
@@ -852,7 +672,6 @@ gucharmap_unichar_type (gunichar uc)
   return G_UNICODE_UNASSIGNED;
 }
 
-
 /**
  * gucharmap_unichar_isdefined:
  * @uc: a Unicode character
@@ -868,7 +687,6 @@ gucharmap_unichar_isdefined (gunichar uc)
   return gucharmap_unichar_type (uc) != G_UNICODE_UNASSIGNED;
 }
 
-
 /**
  * gucharmap_unichar_isgraph:
  * @uc: a Unicode character
